@@ -9,14 +9,27 @@
 # For users who haven't customized the setting, we simply delegate to
 # `DashboardController#show`, which is the default.
 class RootController < Dashboard::ProjectsController
+  include HomepageData
+  include ::Gitlab::InternalEventsTracking
+
   skip_before_action :authenticate_user!, only: [:index]
 
   before_action :redirect_unlogged_user, if: -> { current_user.nil? }
   before_action :redirect_logged_user, if: -> { current_user.present? }
 
+  before_action only: [:index] do
+    push_frontend_feature_flag(:personal_homepage, current_user)
+  end
+
   CACHE_CONTROL_HEADER = 'no-store'
 
-  def index # rubocop:disable Lint/UselessMethodDefinition -- we need to explicitly define this action for the `skip_before_action`
+  def index
+    @homepage_app_data = homepage_app_data(current_user)
+    if Feature.enabled?(:personal_homepage, current_user)
+      track_internal_event('user_views_homepage', user: current_user)
+      render('root/index') && return
+    end
+
     super
   end
 
@@ -35,7 +48,9 @@ class RootController < Dashboard::ProjectsController
   end
 
   def redirect_logged_user
-    case current_user.dashboard
+    case current_user.effective_dashboard_for_routing
+    when 'projects'
+      redirect_to(dashboard_projects_path) if Feature.enabled?(:personal_homepage, current_user)
     when 'stars'
       flash.keep
       redirect_to(starred_dashboard_projects_path)
