@@ -4,8 +4,11 @@ module Packages
   module Composer
     class PackageFinder < ::Packages::GroupOrProjectPackageFinder
       extend ::Gitlab::Utils::Override
+      include Gitlab::Utils::StrongMemoize
 
       def execute
+        return packages if packages_composer_read_from_detached_table?
+
         packages.preload_composer
       end
 
@@ -28,8 +31,17 @@ module Packages
 
       override :packages_class
       def packages_class
-        ::Packages::Composer::Sti::Package
+        if packages_composer_read_from_detached_table?
+          ::Packages::Composer::Package
+        else
+          ::Packages::Composer::Sti::Package
+        end
       end
+
+      def packages_composer_read_from_detached_table?
+        Feature.enabled?(:packages_composer_read_from_detached_table, Feature.current_request)
+      end
+      strong_memoize_attr :packages_composer_read_from_detached_table?
     end
   end
 end
