@@ -11,6 +11,10 @@ module NavHelper
     header_links.include?(link)
   end
 
+  def extra_top_bar_classes
+    # overridden in ee
+  end
+
   def page_with_sidebar_class
     class_name = page_gutter_class
 
@@ -22,6 +26,12 @@ module NavHelper
     class_name
   end
 
+  def super_sidebar_loading_state_class
+    return '' unless project_studio_enabled?
+
+    cookies['super_sidebar_collapsed'] == 'true' ? 'super-sidebar-is-icon-only' : ''
+  end
+
   def page_gutter_class
     if (page_has_markdown? || current_path?('projects/merge_requests#diffs')) && !current_controller?('conflicts')
       if cookies[:collapsed_gutter] == 'true'
@@ -85,13 +95,12 @@ module NavHelper
   end
 
   def work_item_epic_page?
-    current_controller?('epics') && @group.work_item_epics_enabled?
+    current_controller?('epics')
   end
 
   def new_issue_look?
     current_controller?('issues') &&
-      (Feature.enabled?(:work_item_view_for_issues, @project&.group) ||
-      current_user&.user_preference&.use_work_items_view) &&
+      Feature.enabled?(:work_item_view_for_issues, @project&.group) &&
       !@issue&.work_item_type&.incident? &&
       !@issue&.from_service_desk?
   end
