@@ -11,7 +11,6 @@ module SearchHelper
     :snippets,
     :sort,
     :force_search_results,
-    :project_ids,
     :type
   ].freeze
 
@@ -65,8 +64,7 @@ module SearchHelper
   def generic_results(term)
     search_pattern = Regexp.new(Regexp.escape(term), "i")
 
-    generic_results = project_autocomplete + default_autocomplete + help_autocomplete
-    generic_results.concat(default_autocomplete_admin) if current_user.can_read_all_resources?
+    generic_results = combined_generic_results
     generic_results.select { |result| result[:label] =~ search_pattern }
   end
 
@@ -276,6 +274,10 @@ module SearchHelper
 
   private
 
+  def combined_generic_results
+    project_autocomplete + default_autocomplete + help_autocomplete + default_autocomplete_admin
+  end
+
   def formatted_count(scope)
     return "0" if @timeout
 
@@ -293,6 +295,8 @@ module SearchHelper
 
   # Autocomplete results for settings pages, for admins
   def default_autocomplete_admin
+    return [] unless current_user.can_read_all_resources?
+
     [
       { category: "Jump to", label: _("Admin area / Dashboard"), url: admin_root_path }
     ]
@@ -305,8 +309,9 @@ module SearchHelper
       { category: "Help", label: _("Markdown Help"),                url: help_page_path("user/markdown.md") },
       { category: "Help", label: _("Permissions Help"),             url: help_page_path("user/permissions.md") },
       { category: "Help", label: _("Public Access Help"),           url: help_page_path("user/public_access.md") },
-      { category: "Help", label: _("Rake Tasks Help"),              url: help_page_path("raketasks/_index.md") },
-      { category: "Help", label: _("SSH Keys Help"),                url: help_page_path("user/ssh.md") },
+      { category: "Help", label: _("Rake Tasks Help"),
+        url: help_page_path("administration/raketasks/_index.md") },
+      { category: "Help", label: _("SSH Keys Help"), url: help_page_path("user/ssh.md") },
       {
         category: "Help",
         label: s_("Webhooks|System hooks help"),
@@ -471,29 +476,6 @@ module SearchHelper
     Sanitize.clean(str)
   end
 
-  def search_filter_link(scope, label, data: {}, search: {})
-    search_params = params
-      .merge(search)
-      .merge({ scope: scope })
-      .permit(SEARCH_GENERIC_PARAMS)
-
-    if @scope == scope
-      li_class = 'active'
-      count = formatted_count(scope)
-    else
-      badge_class = 'js-search-count hidden'
-      badge_data = { url: search_count_path(search_params) }
-    end
-
-    content_tag :li, class: li_class, data: data do
-      link_to search_path(search_params) do
-        concat label
-        concat ' '
-        concat gl_badge_tag(count, { class: badge_class, data: badge_data })
-      end
-    end
-  end
-
   def active_nav?(active_scope, active_type, type)
     return active_scope unless @scope.to_s == 'issues'
     return active_type if type
@@ -591,7 +573,7 @@ module SearchHelper
   end
 
   def simple_search_highlight_and_truncate(text, phrase, options = {})
-    highlight(search_sanitize(search_truncate(text)), phrase.split, options)
+    highlight(search_sanitize(search_truncate(text)), phrase.split, options) if text.is_a?(String) && !text.empty?
   end
 
   # Sanitize a HTML field for search display. Most tags are stripped out and the
@@ -613,8 +595,6 @@ module SearchHelper
 
   # _search_highlight is used in EE override
   def highlight_and_truncate_issuable(issuable, search_term, _search_highlight)
-    return unless issuable.description.present?
-
     simple_search_highlight_and_truncate(issuable.description, search_term)
   end
 
