@@ -25,6 +25,8 @@ module EncryptedUserPassword
   # Also migrates the user password to the configured
   # encryption type (BCrypt or PBKDF2+SHA512), if needed.
   def valid_password?(password)
+    # On Ubuntu 22.04 FIPS, attempting to hash a password < 8 bytes results in PKCS5_PBKDF2_HMAC: invalid key length
+    return false unless password.length >= ApplicationSetting::DEFAULT_MINIMUM_PASSWORD_LENGTH
     return false unless password_matches?(password)
 
     migrate_password!(password)
@@ -34,6 +36,8 @@ module EncryptedUserPassword
     @password = new_password # rubocop:disable Gitlab/ModuleWithInstanceVariables
     return unless new_password.present?
 
+    return unless new_password.to_s.length >= ApplicationSetting::DEFAULT_MINIMUM_PASSWORD_LENGTH
+
     # Use SafeRequestStore to cache the password hash during registration
     # This prevents redundant bcrypt operations when the same password is set on multiple
     # User objects during registration. Our analysis showed that two separate User objects
