@@ -157,7 +157,7 @@ module Issuable
     attr_mentionable :description
 
     participant :author
-    participant :notes_with_associations
+    participant :notes_for_participants
     participant :assignees
 
     strip_attributes! :title
@@ -648,6 +648,10 @@ module Issuable
     assignees.map(&:username).to_sentence
   end
 
+  def notes_for_participants
+    notes_with_associations.limit(Noteable::MAX_NOTES_LIMIT * 2)
+  end
+
   def notes_with_associations
     # If A has_many Bs, and B has_many Cs, and you do
     # `A.includes(b: :c).each { |a| a.b.includes(:c) }`, sadly ActiveRecord
