@@ -15,6 +15,7 @@ class Member < ApplicationRecord
   include UpdateHighestRole
   include RestrictedSignup
   include Gitlab::Experiment::Dsl
+  include Ci::PipelineScheduleOwnershipValidator
 
   ignore_column :last_activity_on, remove_with: '17.8', remove_after: '2024-12-23'
 
@@ -69,12 +70,32 @@ class Member < ApplicationRecord
   scope :in_hierarchy, ->(source) do
     source = source.root_ancestor
     groups = source.self_and_descendants
-    group_members = Member.default_scoped.where(source: groups).select(*Member.cached_column_list)
-
     projects = source.root_ancestor.all_projects
-    project_members = Member.default_scoped.where(source: projects).select(*Member.cached_column_list)
 
-    Member.default_scoped.from_union([group_members, project_members]).merge(self)
+    group_members = where(source: groups).select(*cached_column_list)
+    project_members = where(source: projects).select(*cached_column_list)
+
+    from_union([group_members, project_members])
+  end
+
+  scope :seat_assignable, ->(users:, namespace: nil) do
+    users = users.is_a?(ActiveRecord::Relation) ? users : Array.wrap(users)
+
+    scope = without_invites_and_requests(minimal_access: true).merge(with_user(users))
+    scope = scope.in_hierarchy(namespace) if namespace
+    scope
+  end
+
+  def self.seat_assignable?(user:, namespace: nil)
+    seat_assignable(users: user, namespace: namespace).exists?
+  end
+
+  def self.seat_assignable_highest_access_level(user:, namespace: nil)
+    seat_assignable(users: user, namespace: namespace).maximum(:access_level)
+  end
+
+  def self.seat_assignable_highest_access_levels(users:, namespace: nil)
+    seat_assignable(users: users, namespace: namespace).group(:user_id).maximum(:access_level)
   end
 
   scope :for_self_and_descendants, ->(group, columns = Member.cached_column_list) do
@@ -244,6 +265,10 @@ class Member < ApplicationRecord
       .order('source_id, source_type, LOWER(invite_email)')
   end
 
+  scope :by_user_types, ->(user_types) do
+    left_join_users.where(users: { user_type: user_types })
+  end
+
   scope :order_name_asc, -> do
     build_keyset_order_on_joined_column(
       scope: left_join_users,
@@ -346,6 +371,7 @@ class Member < ApplicationRecord
   after_save :log_invitation_token_cleanup
 
   after_commit :send_request, if: :request?, unless: :importing?, on: [:create]
+  after_commit :log_previous_state_on_update, unless: :importing?, on: [:update]
   after_commit on: [:create, :update, :destroy], unless: :importing? do
     refresh_member_authorized_projects
   end
@@ -442,17 +468,17 @@ class Member < ApplicationRecord
       select(member_columns_with_no_access)
     end
 
-    def with_group_group_sharing_access(shared_groups, custom_role_for_group_link_enabled)
-      columns = member_columns_with_group_sharing_access(custom_role_for_group_link_enabled)
+    def with_group_group_sharing_access(group)
+      columns = member_columns_with_group_sharing_access(group)
 
       joins("LEFT OUTER JOIN group_group_links ON members.source_id = group_group_links.shared_with_group_id")
         .select(columns)
-        .where(group_group_links: { shared_group_id: shared_groups })
+        .where(group_group_links: { shared_group_id: group.self_and_ancestors })
     end
 
     private
 
-    def member_columns_with_group_sharing_access(custom_role_for_group_link_enabled)
+    def member_columns_with_group_sharing_access(group)
       group_group_link_table = GroupGroupLink.arel_table
 
       column_names.map do |column_name|
@@ -461,7 +487,7 @@ class Member < ApplicationRecord
           args = [group_group_link_table[:group_access], arel_table[:access_level]]
           smallest_value_arel(args, 'access_level')
         when 'member_role_id'
-          member_role_id(group_group_link_table, custom_role_for_group_link_enabled)
+          member_role_id(group)
         else
           arel_table[column_name]
         end
@@ -481,8 +507,8 @@ class Member < ApplicationRecord
     end
 
     # overriden in EE
-    def member_role_id(_group_link_table, _custom_role_for_group_link_enabled)
-      arel_table[:member_role_id]
+    def member_role_id(_group)
+      Arel::Nodes::As.new(Arel::Nodes::SqlLiteral.new('NULL'), Arel::Nodes::SqlLiteral.new('member_role_id'))
     end
   end
 
@@ -628,8 +654,21 @@ class Member < ApplicationRecord
     end
   end
 
-  def prevent_role_assignement?(_current_user, _params)
-    false
+  def prevent_role_assignement?(current_user, params)
+    return false if current_user.can_admin_all_resources?
+
+    # access_level will already be set for accepting access request invites
+    assigning_access_level = params[:access_level] || access_level
+    current_access_level = params[:current_access_level]
+
+    # check if it's a valid downgrade, if the member's current access level encompasses the target level
+    return false if Authz::Role.access_level_encompasses?(
+      current_access_level: current_access_level,
+      level_to_assign: assigning_access_level
+    )
+
+    # prevent assignement in case the role access level is higher than current user's role
+    source.assigning_role_too_high?(current_user, assigning_access_level)
   end
 
   private
@@ -689,6 +728,10 @@ class Member < ApplicationRecord
       run_after_commit { notification_service.updated_member_access_level(self) }
     end
 
+    if pipeline_schedule_ownership_revoked?
+      run_after_commit { notify_unavailable_owned_pipeline_schedules(user.id, source) }
+    end
+
     if saved_change_to_expires_at?
       run_after_commit { notification_service.updated_member_expiration(self) }
     end
@@ -697,6 +740,10 @@ class Member < ApplicationRecord
   end
 
   def post_destroy_member_hook
+    if access_level&.>=(Gitlab::Access::DEVELOPER)
+      run_after_commit { notify_unavailable_owned_pipeline_schedules(user.id, source) }
+    end
+
     system_hook_service.execute_hooks_for(self, :destroy)
   end
 
@@ -704,6 +751,14 @@ class Member < ApplicationRecord
     system_hook_service.execute_hooks_for(self, :revoke)
   end
 
+  def log_previous_state_on_update
+    Gitlab::AppLogger.info(
+      message: 'Refresh member authorized projects on update',
+      user_id: self.user_id,
+      previous_changes: previous_changes.keys
+    )
+  end
+
   # Refreshes authorizations of the current member.
   #
   # This method schedules a job using Sidekiq and as such **must not** be called
