@@ -11,14 +11,15 @@ module Environments
       environment = to_resource(job)
 
       if environment.persisted?
-        if Feature.enabled?(:persisted_job_environment_relationship, job.project)
-          job.run_after_commit do
-            job.link_to_environment(environment)
-          end
+        job.run_after_commit do
+          job.link_to_environment(environment)
         end
 
         job.persisted_environment = environment
-        job.assign_attributes(metadata_attributes: { expanded_environment_name: environment.name })
+
+        if Feature.disabled?(:stop_writing_builds_metadata, job.project)
+          job.assign_attributes(metadata_attributes: { expanded_environment_name: environment.name })
+        end
 
         track_environment_usage(job, environment)
       else
@@ -36,7 +37,7 @@ module Environments
                        .safe_find_or_create_by(name: job.expanded_environment_name) do |environment|
         # Initialize the attributes at creation
         environment.auto_stop_in = expanded_auto_stop_in(job)
-        environment.tier = job.environment_tier_from_options
+        environment.tier = job.expanded_deployment_tier
         environment.merge_request = job.pipeline.merge_request
       end
 
