@@ -2,7 +2,7 @@
 
 module Users
   class BuildService < BaseService
-    ALLOWED_USER_TYPES = %i[project_bot security_policy_bot].freeze
+    ALLOWED_USER_TYPES = %i[project_bot security_policy_bot placeholder].freeze
 
     delegate :user_default_internal_regex_enabled?,
       :user_default_internal_regex_instance,
@@ -88,13 +88,14 @@ module Users
     def organization_access_level
       return organization_params[:organization_access_level] if organization_params.has_key?(:organization_access_level)
 
-      Organizations::OrganizationUser.default_organization_access_level(user_is_admin: @user.admin?)
+      Organizations::OrganizationUser.home_organization_access_level(user_is_admin: @user.admin?)
     end
 
     def assign_organization
       # Allow invalid parameters for the validation errors to bubble up to the User.
       return if organization_params.blank?
 
+      @user.organization_id = organization_params[:organization_id]
       @user.organization_users << Organizations::OrganizationUser.new(
         organization_id: organization_params[:organization_id],
         access_level: organization_access_level
@@ -109,6 +110,7 @@ module Users
     def assign_common_user_params
       @user_params[:created_by_id] = current_user&.id
       @user_params[:external] = user_external? if set_external_param?
+      @user_params[:email_otp_required_after] = Time.current if enrol_new_users_in_email_otp?
 
       @user_params.delete(:user_type) unless allowed_user_type?
     end
@@ -117,6 +119,10 @@ module Users
       user_default_internal_regex_enabled? && !user_params.key?(:external)
     end
 
+    def enrol_new_users_in_email_otp?
+      Feature.enabled?(:enrol_new_users_in_email_otp, :instance)
+    end
+
     def user_external?
       user_default_internal_regex_instance.match(params[:email]).nil?
     end
@@ -195,7 +201,7 @@ module Users
         :location,
         :name,
         :note,
-        :organization,
+        :user_detail_organization,
         :password,
         :password_automatically_set,
         :password_expires_at,
@@ -204,13 +210,13 @@ module Users
         :public_email,
         :remember_me,
         :skip_confirmation,
-        :skype,
         :theme_id,
         :twitter,
         :user_type,
         :username,
         :view_diffs_file_by_file,
-        :website_url
+        :website_url,
+        :github
       ]
     end
 
