@@ -266,6 +266,9 @@ module Gitlab
     # Disable adding field_with_errors wrapper to form elements
     config.action_view.field_error_proc = proc { |html_tag, instance| html_tag }
 
+    # Enable Sendfile middleware to offload serving of files to Workhorse
+    config.action_dispatch.x_sendfile_header = 'X-Sendfile'
+
     # Support legacy unicode file named img emojis, `1F939.png`
     config.assets.paths << TanukiEmoji.images_path
     config.assets.paths << "#{config.root}/vendor/assets/fonts"
