@@ -143,6 +143,8 @@ module API
       end
 
       resource :jobs do
+        helpers ::API::Helpers::RateLimiter
+
         before { set_application_context }
 
         desc 'Request a job' do
@@ -151,7 +153,8 @@ module API
             [204, 'No job for Runner'],
             [403, 'Forbidden'],
             [409, 'Conflict'],
-            [422, 'Runner is orphaned']]
+            [422, 'Runner is orphaned'],
+            [429, 'Too Many Requests']]
         end
         params do
           requires :token, type: String, desc: "Runner's authentication token"
@@ -191,6 +194,8 @@ module API
         parser :build_json, ::Grape::Parser::Json
 
         post '/request', urgency: :low, feature_category: :continuous_integration do
+          check_rate_limit!(:runner_jobs_request_api, scope: [Gitlab::CryptoHelper.sha256(params[:token])], user: nil)
+
           authenticate_runner!(creation_state: :finished)
 
           unless current_runner.active?
@@ -230,12 +235,14 @@ module API
           http_codes [[200, 'Job was updated'],
             [202, 'Update accepted'],
             [400, 'Unknown parameters'],
-            [403, 'Forbidden']]
+            [403, 'Forbidden'],
+            [409, 'Conflict'],
+            [429, 'Too Many Requests']]
         end
         params do
-          requires :token, type: String, desc: 'Job token'
+          requires :token, type: String, desc: "Job's authentication token"
           requires :id, type: Integer, desc: "Job's ID"
-          optional :state, type: String, desc: "Job's status: success, failed"
+          optional :state, type: String, desc: "Job's status: pending, running, success, failed"
           optional :checksum, type: String, desc: "Job's trace CRC32 checksum"
           optional :failure_reason, type: String, desc: "Job's failure_reason"
           optional :output, type: Hash, desc: 'Build log state' do
@@ -245,12 +252,14 @@ module API
           optional :exit_code, type: Integer, desc: "Job's exit code"
         end
         put '/:id', urgency: :low, feature_category: :continuous_integration do
+          check_rate_limit!(:runner_jobs_api, scope: [Gitlab::CryptoHelper.sha256(job_token)], user: nil)
+
           job = authenticate_job!(heartbeat_runner: true)
 
           Gitlab::Metrics.add_event(:update_build)
 
-          service = ::Ci::UpdateBuildStateService
-            .new(job, declared_params(include_missing: false))
+          # Handle job state updates through the service (including two-phase commit workflow)
+          service = ::Ci::UpdateBuildStateService.new(job, declared_params(include_missing: false))
 
           service.execute.then do |result|
             track_ci_minutes_usage!(job)
@@ -266,14 +275,17 @@ module API
           http_codes [[202, 'Trace was patched'],
             [400, 'Missing Content-Range header'],
             [403, 'Forbidden'],
-            [416, 'Range not satisfiable']]
+            [416, 'Range not satisfiable'],
+            [429, 'Too Many Requests']]
         end
         params do
           requires :id, type: Integer, desc: "Job's ID"
-          optional :token, type: String, desc: "Job's authentication token"
-          optional :debug_trace, type: Boolean, desc: 'Enable or Disable the debug trace'
+          optional :token, type: String, desc: "Job's authentication token" # token can also be present in header
+          optional :debug_trace, type: Boolean, desc: 'Enable or disable the debug trace'
         end
         patch '/:id/trace', urgency: :low, feature_category: :continuous_integration do
+          check_rate_limit!(:runner_jobs_patch_trace_api, scope: [Gitlab::CryptoHelper.sha256(job_token)], user: nil)
+
           job = authenticate_job!(heartbeat_runner: true)
 
           error!('400 Missing header Content-Range', 400) unless request.headers.key?('Content-Range')
@@ -304,11 +316,12 @@ module API
           http_codes [[200, 'Upload allowed'],
             [403, 'Forbidden'],
             [405, 'Artifacts support not enabled'],
-            [413, 'File too large']]
+            [413, 'File too large'],
+            [429, 'Too Many Requests']]
         end
         params do
           requires :id, type: Integer, desc: "Job's ID"
-          optional :token, type: String, desc: "Job's authentication token"
+          optional :token, type: String, desc: "Job's authentication token" # token can also be present in header
 
           # NOTE:
           # In current runner, filesize parameter would be empty here. This is because archive is streamed by runner,
@@ -320,6 +333,8 @@ module API
             default: 'archive', values: ::Ci::JobArtifact.file_types.keys
         end
         post '/:id/artifacts/authorize', feature_category: :job_artifacts, urgency: :low do
+          check_rate_limit!(:runner_jobs_api, scope: [Gitlab::CryptoHelper.sha256(job_token)], user: nil)
+
           not_allowed! unless Gitlab.config.artifacts.enabled
           require_gitlab_workhorse!
 
@@ -342,12 +357,13 @@ module API
             [400, 'Bad request'],
             [403, 'Forbidden'],
             [405, 'Artifacts support not enabled'],
-            [413, 'File too large']]
+            [413, 'File too large'],
+            [429, 'Too Many Requests']]
         end
         params do
           requires :id, type: Integer, desc: "Job's ID"
           requires :file, type: ::API::Validations::Types::WorkhorseFile, desc: "The artifact file to store (generated by Multipart middleware)", documentation: { type: 'file' }
-          optional :token, type: String, desc: "Job's authentication token"
+          optional :token, type: String, desc: "Job's authentication token" # token can also be present in header
           optional :expire_in, type: String, desc: 'Specify when artifact should expire'
           optional :artifact_type, type: String, desc: 'The type of artifact',
             default: 'archive', values: ::Ci::JobArtifact.file_types.keys
@@ -357,6 +373,8 @@ module API
           optional :accessibility, type: String, desc: 'Specify accessibility level of artifact private/public'
         end
         post '/:id/artifacts', feature_category: :job_artifacts, urgency: :low do
+          check_rate_limit!(:runner_jobs_api, scope: [Gitlab::CryptoHelper.sha256(job_token)], user: nil)
+
           not_allowed! unless Gitlab.config.artifacts.enabled
           require_gitlab_workhorse!
 
@@ -382,17 +400,20 @@ module API
             [302, 'Found'],
             [401, 'Unauthorized'],
             [403, 'Forbidden'],
-            [404, 'Artifact not found']]
+            [404, 'Artifact not found'],
+            [429, 'Too Many Requests']]
         end
         params do
           requires :id, type: Integer, desc: "Job's ID"
-          optional :token, type: String, desc: "Job's authentication token"
+          optional :token, type: String, desc: "Job's authentication token" # token can also be present in header
           optional :direct_download, default: false, type: Boolean, desc: 'Perform direct download from remote storage instead of proxying artifacts'
         end
         route_setting :authentication, job_token_allowed: true
         route_setting :authorization, job_token_policies: :read_jobs,
           allow_public_access_for_enabled_project_features: [:repository, :builds]
         get '/:id/artifacts', feature_category: :job_artifacts do
+          check_rate_limit!(:runner_jobs_api, scope: [Gitlab::CryptoHelper.sha256(job_token)], user: nil)
+
           authenticate_job_via_dependent_job!
           authorize_job_token_policies!(current_job.project)
 
