@@ -63,8 +63,6 @@ module API
         end
 
         def authenticate_show_runner!(runner)
-          return if runner.instance_type? || current_user.can_read_all_resources?
-
           forbidden!("No access granted") unless can?(current_user, :read_runner, runner)
         end
 
@@ -90,6 +88,15 @@ module API
           forbidden!("No access granted") unless can?(current_user, :assign_runner, runner)
         end
 
+        def authenticate_disable_runner!(runner_project)
+          not_found!('Runner') unless runner_project
+
+          return if current_user.can_admin_all_resources?
+
+          forbidden!("Runner is locked") if runner_project.runner.locked?
+          forbidden!("No access granted") unless can?(current_user, :unassign_runner, runner_project)
+        end
+
         def authenticate_list_runners_jobs!(runner)
           return if current_user.can_read_all_resources?
 
@@ -112,7 +119,7 @@ module API
 
       resource :runners do
         desc 'Get runners available for user' do
-          summary 'List owned runners'
+          summary 'List available runners'
           success Entities::Ci::Runner
           failure [[400, 'Scope contains invalid value'], [401, 'Unauthorized']]
           tags %w[runners]
@@ -122,11 +129,11 @@ module API
           use :filter_params
         end
         get do
-          runners = current_user.ci_owned_runners
+          runners = current_user.ci_available_runners.with_api_entity_associations
           runners = filter_runners(runners, params[:scope], allowed_scopes: ::Ci::Runner::AVAILABLE_STATUSES_INCL_DEPRECATED)
           runners = apply_filter(runners, params)
 
-          present paginate(runners), with: Entities::Ci::Runner
+          present paginate(runners), with: Entities::Ci::Runner, current_user: current_user
         end
 
         desc 'Get all runners - shared and project' do
@@ -144,11 +151,11 @@ module API
         get 'all' do
           authenticated_with_can_read_all_resources!
 
-          runners = ::Ci::Runner.all
+          runners = ::Ci::Runner.all.with_api_entity_associations
           runners = filter_runners(runners, params[:scope])
           runners = apply_filter(runners, params)
 
-          present paginate(runners), with: Entities::Ci::Runner
+          present paginate(runners), with: Entities::Ci::Runner, current_user: current_user
         end
 
         desc "Get runner's details" do
@@ -221,8 +228,8 @@ module API
           summary 'Delete a runner'
           success Entities::Ci::Runner
           failure [[401, 'Unauthorized'], [403, 'No access granted'],
-                   [403, 'Runner associated with more than one project'], [404, 'Runner not found'],
-                   [412, 'Precondition Failed']]
+            [403, 'Runner associated with more than one project'], [404, 'Runner not found'],
+            [412, 'Precondition Failed']]
           tags %w[runners]
         end
         params do
@@ -294,7 +301,7 @@ module API
           desc: 'The ID or URL-encoded path of the project owned by the authenticated user'
       end
       resource :projects, requirements: API::NAMESPACE_OR_PROJECT_REQUIREMENTS do
-        before { authorize! :read_project_runners, user_project }
+        before { authorize! :read_runners, user_project }
 
         desc 'Get runners available for project' do
           summary "List project's runners"
@@ -309,35 +316,35 @@ module API
           use :filter_params
         end
         get ':id/runners' do
-          runners = ::Ci::Runner.owned_or_instance_wide(user_project.id)
+          runners = ::Ci::Runner.owned_or_instance_wide(user_project.id).with_api_entity_associations
           # scope is deprecated (for project runners), however api documentation still supports it.
           # Not including them in `apply_filter` method as it's not supported for group runners
           runners = filter_runners(runners, params[:scope])
           runners = apply_filter(runners, params)
 
-          present paginate(runners), with: Entities::Ci::Runner
+          present paginate(runners), with: Entities::Ci::Runner, current_user: current_user
         end
 
         desc 'Assign a runner to project' do
           detail "Assign an available project runner to the project."
           success Entities::Ci::Runner
           failure [[400, 'Bad Request'],
-                   [403, 'No access granted'], [403, 'Runner is a group runner'], [403, 'Runner is locked'],
-                   [404, 'Runner not found']]
+            [403, 'No access granted'], [403, 'Runner is a group runner'], [403, 'Runner is locked'],
+            [404, 'Runner not found']]
           tags %w[runners projects]
         end
         params do
           requires :runner_id, type: Integer, desc: 'The ID of a runner'
         end
         post ':id/runners' do
-          authorize! :create_runner, user_project # Ensure the user is allowed to create a runner on the target project
+          authorize! :create_runners, user_project # Ensure the user is allowed to create a runner on the target project
 
           runner = get_runner(params[:runner_id])
           authenticate_enable_runner!(runner)
 
           result = ::Ci::Runners::AssignRunnerService.new(runner, user_project, current_user).execute
           if result.success?
-            present runner, with: Entities::Ci::Runner
+            present runner, with: Entities::Ci::Runner, current_user: current_user
           else
             render_api_error_with_reason!(:bad_request, result.message, result.reason)
           end
@@ -349,18 +356,16 @@ module API
                  "If so, an error is returned. Use the call to delete a runner instead."
           success Entities::Ci::Runner
           failure [[400, 'Bad Request'],
-                   [403, 'You cannot unassign a runner from the owner project. Delete the runner instead'],
-                   [404, 'Runner not found'], [412, 'Precondition Failed']]
+            [403, 'You cannot unassign a runner from the owner project. Delete the runner instead'],
+            [404, 'Runner not found'], [412, 'Precondition Failed']]
           tags %w[runners projects]
         end
         params do
           requires :runner_id, type: Integer, desc: 'The ID of a runner'
         end
         delete ':id/runners/:runner_id' do
-          authorize! :admin_project_runners, user_project
-
           runner_project = user_project.runner_projects.find_by_runner_id(params[:runner_id])
-          not_found!('Runner') unless runner_project
+          authenticate_disable_runner!(runner_project)
 
           destroy_conditionally!(runner_project) do
             response = ::Ci::Runners::UnassignRunnerService.new(runner_project, current_user).execute
@@ -373,7 +378,7 @@ module API
         requires :id, type: String, desc: 'The ID of a group'
       end
       resource :groups, requirements: API::NAMESPACE_OR_PROJECT_REQUIREMENTS do
-        before { authorize! :read_group_runners, user_group }
+        before { authorize! :read_runners, user_group }
 
         desc 'Get runners available for group' do
           summary "List group's runners"
@@ -387,10 +392,10 @@ module API
           use :filter_params
         end
         get ':id/runners' do
-          runners = ::Ci::Runner.group_or_instance_wide(user_group)
+          runners = ::Ci::Runner.group_or_instance_wide(user_group).with_api_entity_associations
           runners = apply_filter(runners, params)
 
-          present paginate(runners), with: Entities::Ci::Runner
+          present paginate(runners), with: Entities::Ci::Runner, current_user: current_user
         end
       end
 
