@@ -53,7 +53,9 @@ module API
       end
 
       def authorize_ai_access!
-        forbidden!('Insufficient permissions for Duo Workflow') unless can?(current_user, :duo_workflow, user_project)
+        return if can?(current_user, :duo_workflow, user_project) || can?(current_user, :access_duo_agentic_chat, user_project)
+
+        forbidden!('Insufficient permissions for Duo Agent Platform')
       end
 
       def commit_response(attrs)
@@ -111,7 +113,7 @@ module API
 
       params :simple_file_params do
         requires :file_path, type: String, file_path: true,
-          desc: 'The url encoded path to the file.', documentation: { example: 'lib%2Fclass%2Erb' }
+          desc: 'The URL-encoded path to the file.', documentation: { example: 'lib%2Fclass%2Erb' }
         requires :branch, type: String,
           desc: 'Name of the branch to commit into. To create a new branch, also provide `start_branch`.', allow_blank: false,
           documentation: { example: 'main' }
@@ -143,7 +145,7 @@ module API
       desc 'Get blame file metadata from repository'
       params do
         requires :file_path, type: String, file_path: true,
-          desc: 'The url encoded path to the file.', documentation: { example: 'lib%2Fclass%2Erb' }
+          desc: 'The URL-encoded path to the file.', documentation: { example: 'lib%2Fclass%2Erb' }
         requires :ref, type: String,
           desc: 'The name of branch, tag or commit', allow_blank: false, documentation: { example: 'main' }
       end
@@ -156,10 +158,10 @@ module API
       desc 'Get blame file from the repository'
       params do
         requires :file_path, type: String, file_path: true,
-          desc: 'The url encoded path to the file.', documentation: { example: 'lib%2Fclass%2Erb' }
+          desc: 'The URL-encoded path to the file.', documentation: { example: 'lib%2Fclass%2Erb' }
         requires :ref, type: String,
           desc: 'The name of branch, tag or commit', allow_blank: false, documentation: { example: 'main' }
-        optional :range, type: Hash do
+        optional :range, type: Hash, desc: 'Object that contains the blame range' do
           requires :start, type: Integer,
             desc: 'The first line of the range to blame', allow_blank: false, values: ->(v) { v > 0 }
           requires :end, type: Integer,
@@ -181,14 +183,17 @@ module API
         success File
       end
       params do
-        requires :file_path, type: String, file_path: true,
-          desc: 'The url encoded path to the file.', documentation: { example: 'lib%2Fclass%2Erb' }
+        requires :file_path, type: String, file_path: { allow_initial_path_separator: true },
+          desc: 'The URL-encoded path to the file.', documentation: { example: 'lib%2Fclass%2Erb' }
         optional :ref, type: String,
           desc: 'The name of branch, tag or commit', allow_blank: false, documentation: { example: 'main' }
         optional :lfs, type: Boolean,
           desc: 'Retrieve binary data for a file that is an lfs pointer',
           default: false
       end
+      route_setting :authentication, job_token_allowed: true
+      route_setting :authorization, job_token_policies: :read_repositories,
+        allow_public_access_for_enabled_project_features: :repository
       get ":id/repository/files/:file_path/raw", requirements: FILE_ENDPOINT_REQUIREMENTS, urgency: :low do
         assign_file_vars!
 
@@ -208,7 +213,7 @@ module API
       desc 'Get file metadata from repository'
       params do
         requires :file_path, type: String, file_path: true,
-          desc: 'The url encoded path to the file.', documentation: { example: 'lib%2Fclass%2Erb' }
+          desc: 'The URL-encoded path to the file.', documentation: { example: 'lib%2Fclass%2Erb' }
         requires :ref, type: String,
           desc: 'The name of branch, tag or commit', allow_blank: false, documentation: { example: 'main' }
       end
@@ -221,7 +226,7 @@ module API
       desc 'Get a file from the repository'
       params do
         requires :file_path, type: String, file_path: true,
-          desc: 'The url encoded path to the file.', documentation: { example: 'lib%2Fclass%2Erb' }
+          desc: 'The URL-encoded path to the file.', documentation: { example: 'lib%2Fclass%2Erb' }
         requires :ref, type: String,
           desc: 'The name of branch, tag or commit', allow_blank: false, documentation: { example: 'main' }
       end
