@@ -9,8 +9,12 @@ module API
           @manager = manager
         end
 
-        def invoke
-          tools = manager.tools.map do |name, tool|
+        def invoke(current_user)
+          tools_hash = manager.tools
+
+          tools = tools_hash.filter_map do |name, tool|
+            next nil if exclude_tool?(name, current_user)
+
             {
               name: name,
               description: tool.description,
@@ -23,6 +27,12 @@ module API
 
         private
 
+        def exclude_tool?(tool_name, current_user)
+          return Feature.disabled?(:code_snippet_search_graphqlapi, current_user) if tool_name == 'get_code_context'
+
+          false
+        end
+
         attr_reader :manager
       end
     end
