@@ -7,6 +7,9 @@ module API
     include Helpers::CustomAttributes
 
     allow_access_with_scope :read_user, if: ->(request) { request.get? || request.head? }
+    allow_access_with_scope :ai_workflows, if: ->(request) do
+      request.head? || request_current_user?(request)
+    end
 
     feature_category :user_profile,
       %w[
@@ -21,6 +24,10 @@ module API
         /users/:id/custom_attributes/:key
       ]
 
+    def self.request_current_user?(request)
+      request.get? && request.path.match?(%r{/api/v\d+/user$})
+    end
+
     resource :users, requirements: { uid: /[0-9]*/, id: /[0-9]*/ } do
       include CustomAttributesEndpoints
 
@@ -54,11 +61,11 @@ module API
         # rubocop: enable CodeReuse/ActiveRecord
 
         params :optional_attributes do
-          optional :skype, type: String, desc: 'The Skype username'
           optional :linkedin, type: String, desc: 'The LinkedIn username'
           optional :twitter, type: String, desc: 'The Twitter username'
           optional :discord, type: String, desc: 'The Discord user ID'
           optional :website_url, type: String, desc: 'The website of the user'
+          optional :github, type: String, desc: 'The GitHub username'
           optional :organization, type: String, desc: 'The organization of the user'
           optional :projects_limit, type: Integer, desc: 'The number of projects a user can create'
           optional :extern_uid, type: String, desc: 'The external authentication provider UID'
@@ -78,6 +85,7 @@ module API
           optional :private_profile, type: Boolean, desc: 'Flag indicating the user has a private profile'
           optional :note, type: String, desc: 'Admin note for this user'
           optional :view_diffs_file_by_file, type: Boolean, desc: 'Flag indicating the user sees only one file diff per page'
+          optional :policy_advanced_editor, type: Boolean, desc: 'Flag indicating that advanced editor is enabled'
           all_or_none_of :extern_uid, :provider
 
           use :optional_params_ee
@@ -166,6 +174,7 @@ module API
         # CE
         optional :username, type: String, desc: 'Get a single user with a specific username'
         optional :extern_uid, type: String, desc: 'Get a single user with a specific external authentication provider UID'
+        optional :public_email, type: String, desc: 'Get a single user with a specific public email'
         optional :provider, type: String, desc: 'The external provider'
         optional :search, type: String, desc: 'Search for a username'
         optional :active, type: Boolean, default: false, desc: 'Filters only active users'
@@ -191,6 +200,10 @@ module API
       end
       # rubocop: disable CodeReuse/ActiveRecord
       get feature_category: :user_profile, urgency: :low do
+        # This error can be removed in/after 19.0 release.
+        # https://gitlab.com/gitlab-org/gitlab/-/issues/549951
+        error_for_saml_provider_id_param_ee
+
         index_params = declared_params(include_missing: false)
 
         authenticated_as_admin! if index_params[:extern_uid].present? && index_params[:provider].present?
@@ -262,9 +275,7 @@ module API
         requires :user_id, type: String, desc: 'The ID or username of the user'
       end
       get ":user_id/status", requirements: API::USER_REQUIREMENTS, feature_category: :user_profile, urgency: :default do
-        if Feature.enabled?(:rate_limiting_user_endpoints, ::Feature.current_request)
-          check_rate_limit_by_user_or_ip!(:user_status)
-        end
+        check_rate_limit_by_user_or_ip!(:user_status)
 
         user = find_user(params[:user_id])
 
@@ -326,11 +337,7 @@ module API
       get ':id/following', feature_category: :user_profile do
         forbidden!('Not authorized!') unless current_user
 
-        unless current_user.can_read_all_resources?
-          if Feature.enabled?(:rate_limiting_user_endpoints, ::Feature.current_request)
-            check_rate_limit_by_user_or_ip!(:user_following)
-          end
-        end
+        check_rate_limit_by_user_or_ip!(:user_following) unless current_user.can_read_all_resources?
 
         user = find_user(params[:id])
         not_found!('User') unless user && can?(current_user, :read_user_profile, user)
@@ -348,11 +355,7 @@ module API
       get ':id/followers', feature_category: :user_profile do
         forbidden!('Not authorized!') unless current_user
 
-        unless current_user.can_read_all_resources?
-          if Feature.enabled?(:rate_limiting_user_endpoints, ::Feature.current_request)
-            check_rate_limit_by_user_or_ip!(:user_followers)
-          end
-        end
+        check_rate_limit_by_user_or_ip!(:user_followers) unless current_user.can_read_all_resources?
 
         user = find_user(params[:id])
         not_found!('User') unless user && can?(current_user, :read_user_profile, user)
@@ -377,7 +380,7 @@ module API
       post feature_category: :user_profile do
         authenticated_as_admin!
 
-        params = declared_params(include_missing: false).merge(organization_id: Current.organization&.id)
+        params = declared_params(include_missing: false).merge(organization_id: Current.organization.id)
 
         # TODO: Remove in 16.0. Issue: https://gitlab.com/gitlab-org/gitlab/-/issues/387005
         if params.key?(:private_profile) && params[:private_profile].nil?
@@ -416,20 +419,18 @@ module API
         optional :username, type: String, desc: 'The username of the user'
         use :optional_attributes
       end
-      # rubocop: disable CodeReuse/ActiveRecord
+
       put ":id", feature_category: :user_profile do
         authenticated_as_admin!
 
-        user = User.find_by(id: params.delete(:id))
+        user = User.find_by_id(params.delete(:id))
         not_found!('User') unless user
 
         conflict!('Email has already been taken') if params[:email] &&
-          User.by_any_email(params[:email].downcase)
-              .where.not(id: user.id).exists?
+          User.by_any_email(params[:email].downcase).id_not_in(user.id).exists?
 
         conflict!('Username has already been taken') if params[:username] &&
-          User.by_username(params[:username])
-              .where.not(id: user.id).exists?
+          User.by_username(params[:username]).id_not_in(user.id).exists?
 
         user_params = declared_params(include_missing: false)
 
@@ -444,6 +445,8 @@ module API
           user_params[:password_expires_at] = Time.current if admin_making_changes_for_another_user
         end
 
+        user_params[:user_detail_organization] = user_params.delete(:organization) if user_params[:organization]
+
         result = ::Users::UpdateService.new(current_user, user_params.merge(user: user)).execute do |user|
           user.send_only_admin_changed_your_password_notification! if admin_making_changes_for_another_user
         end
@@ -455,7 +458,6 @@ module API
           render_validation_error!(user)
         end
       end
-      # rubocop: enable CodeReuse/ActiveRecord
 
       desc "Disable two factor authentication for a user. Available only for admins" do
         detail 'This feature was added in GitLab 15.2'
@@ -563,9 +565,7 @@ module API
         use :pagination
       end
       get ':user_id/keys', requirements: API::USER_REQUIREMENTS, feature_category: :system_access do
-        if Feature.enabled?(:rate_limiting_user_endpoints, ::Feature.current_request)
-          check_rate_limit_by_user_or_ip!(:user_ssh_keys)
-        end
+        check_rate_limit_by_user_or_ip!(:user_ssh_keys)
 
         user = find_user(params[:user_id])
         not_found!('User') unless user && can?(current_user, :read_user, user)
@@ -582,9 +582,7 @@ module API
         requires :key_id, type: Integer, desc: 'The ID of the SSH key'
       end
       get ':id/keys/:key_id', requirements: API::USER_REQUIREMENTS, feature_category: :system_access do
-        if Feature.enabled?(:rate_limiting_user_endpoints, ::Feature.current_request)
-          check_rate_limit_by_user_or_ip!(:user_ssh_key)
-        end
+        check_rate_limit_by_user_or_ip!(:user_ssh_key)
 
         user = find_user(params[:id])
         not_found!('User') unless user && can?(current_user, :read_user, user)
@@ -654,9 +652,7 @@ module API
       end
       # rubocop: disable CodeReuse/ActiveRecord
       get ':id/gpg_keys', feature_category: :system_access do
-        if Feature.enabled?(:rate_limiting_user_endpoints, ::Feature.current_request)
-          check_rate_limit_by_user_or_ip!(:user_gpg_keys)
-        end
+        check_rate_limit_by_user_or_ip!(:user_gpg_keys)
 
         user = User.find_by(id: params[:id])
         not_found!('User') unless user
@@ -675,9 +671,7 @@ module API
       end
       # rubocop: disable CodeReuse/ActiveRecord
       get ':id/gpg_keys/:key_id', feature_category: :system_access do
-        if Feature.enabled?(:rate_limiting_user_endpoints, ::Feature.current_request)
-          check_rate_limit_by_user_or_ip!(:user_gpg_key)
-        end
+        check_rate_limit_by_user_or_ip!(:user_gpg_key)
 
         user = User.find_by(id: params[:id])
         not_found!('User') unless user
@@ -812,7 +806,10 @@ module API
 
         user = User.find_by(id: params[:id])
         not_found!('User') unless user
-        conflict!('User cannot be removed while is the sole-owner of a group') unless user.can_be_removed? || params[:hard_delete]
+
+        unless user.can_be_removed? || params[:hard_delete]
+          conflict!('User cannot be removed while is the sole-owner of a group')
+        end
 
         destroy_conditionally!(user) do
           user.delete_async(deleted_by: current_user, params: params)
@@ -979,7 +976,7 @@ module API
       end
       params do
         requires :user_id, type: Integer, desc: 'The ID of the user'
-        optional :type, type: String, values: %w[Project Namespace]
+        optional :type, type: String, desc: 'Filter memberships by type', values: %w[Project Namespace]
         use :pagination
       end
       get ":user_id/memberships", feature_category: :user_profile, urgency: :high do
@@ -1122,7 +1119,7 @@ module API
           end
           post feature_category: :system_access do
             response = ::PersonalAccessTokens::CreateService.new(
-              current_user: current_user, target_user: target_user, organization_id: Current.organization&.id, params: declared_params(include_missing: false)
+              current_user: current_user, target_user: target_user, organization_id: Current.organization.id, params: declared_params(include_missing: false)
             ).execute
 
             if response.success?
@@ -1413,7 +1410,8 @@ module API
         optional :view_diffs_file_by_file, type: Boolean, desc: 'Flag indicating the user sees only one file diff per page'
         optional :show_whitespace_in_diffs, type: Boolean, desc: 'Flag indicating the user sees whitespace changes in diffs'
         optional :pass_user_identities_to_ci_jwt, type: Boolean, desc: 'Flag indicating the user passes their external identities to a CI job as part of a JSON web token.'
-        at_least_one_of :view_diffs_file_by_file, :show_whitespace_in_diffs, :pass_user_identities_to_ci_jwt
+        optional :policy_advanced_editor, type: Boolean, desc: 'Flag indicating that advanced editor is enabled.'
+        at_least_one_of :view_diffs_file_by_file, :show_whitespace_in_diffs, :pass_user_identities_to_ci_jwt, :policy_advanced_editor
       end
       put "preferences", feature_category: :user_profile, urgency: :high do
         authenticate!
@@ -1575,15 +1573,16 @@ module API
         end
         params do
           use :create_personal_access_token_params
-          # NOTE: for security reasons only the k8s_proxy scope is allowed at the moment.
+          # NOTE: for security reasons only the k8s_proxy and self_rotate scope is allowed at the moment.
           # See details in https://gitlab.com/gitlab-org/gitlab/-/merge_requests/131923#note_1571272897
           # and in https://gitlab.com/gitlab-org/gitlab/-/issues/425171
-          requires :scopes, type: Array[String], coerce_with: ::API::Validations::Types::CommaSeparatedToArray.coerce, values: [::Gitlab::Auth::K8S_PROXY_SCOPE].map(&:to_s),
+          # and in https://gitlab.com/gitlab-org/gitlab/-/issues/555546
+          requires :scopes, type: Array[String], coerce_with: ::API::Validations::Types::CommaSeparatedToArray.coerce, values: [::Gitlab::Auth::K8S_PROXY_SCOPE, ::Gitlab::Auth::SELF_ROTATE_SCOPE].map(&:to_s),
             desc: 'The array of scopes of the personal access token'
         end
         post feature_category: :system_access do
           response = ::PersonalAccessTokens::CreateService.new(
-            current_user: current_user, target_user: current_user, params: declared_params(include_missing: false), organization_id: Current.organization&.id
+            current_user: current_user, target_user: current_user, params: declared_params(include_missing: false), organization_id: Current.organization.id
           ).execute
 
           if response.success?
