@@ -16,12 +16,16 @@ module Gitlab
     READ_USER_SCOPE = :read_user
     CREATE_RUNNER_SCOPE = :create_runner
     MANAGE_RUNNER_SCOPE = :manage_runner
+    MCP_SCOPE = :mcp
+    GRANULAR_SCOPE = :granular
     API_SCOPES = [
       API_SCOPE, READ_API_SCOPE,
       READ_USER_SCOPE,
       CREATE_RUNNER_SCOPE, MANAGE_RUNNER_SCOPE,
       K8S_PROXY_SCOPE,
-      SELF_ROTATE_SCOPE
+      SELF_ROTATE_SCOPE,
+      MCP_SCOPE,
+      GRANULAR_SCOPE
     ].freeze
 
     # Scopes for Duo
@@ -76,8 +80,35 @@ module Gitlab
     # Default scopes for OAuth applications that don't define their own
     DEFAULT_SCOPES = [API_SCOPE].freeze
 
+    # Scopes ordered by permission level, from lowest to highest.
+    # This is used to determine the order of scopes in the UI.
+    # If the scope is not present in this list, it won't appear in the UI.
+    UI_SCOPES_ORDERED_BY_PERMISSION = [
+      READ_SERVICE_PING_SCOPE,
+      READ_USER_SCOPE,
+      READ_REPOSITORY_SCOPE,
+      READ_OBSERVABILITY_SCOPE,
+      READ_VIRTUAL_REGISTRY_SCOPE,
+      READ_REGISTRY_SCOPE,
+      READ_API_SCOPE,
+      SELF_ROTATE_SCOPE,
+      WRITE_REPOSITORY_SCOPE,
+      WRITE_OBSERVABILITY_SCOPE,
+      WRITE_VIRTUAL_REGISTRY_SCOPE,
+      WRITE_REGISTRY_SCOPE,
+      API_SCOPE,
+      AI_FEATURES,
+      CREATE_RUNNER_SCOPE,
+      MANAGE_RUNNER_SCOPE,
+      K8S_PROXY_SCOPE,
+      ADMIN_MODE_SCOPE,
+      SUDO_SCOPE
+    ].freeze
+
     CI_JOB_USER = 'gitlab-ci-token'
 
+    extend Gitlab::InternalEventsTracking
+
     class << self
       prepend_mod_with('Gitlab::Auth') # rubocop: disable Cop/InjectEnterpriseEditionModule
 
@@ -102,7 +133,7 @@ module Gitlab
           oauth_access_token_check(password) ||
           personal_access_token_check(password, project) ||
           deploy_token_check(login, password, project) ||
-          user_with_password_for_git(login, password) ||
+          user_with_password_for_git(login, password, request: request) ||
           Gitlab::Auth::Result::EMPTY
 
         rate_limit!(rate_limiter, success: result.success?, login: login, request: request)
@@ -123,7 +154,7 @@ module Gitlab
       # different mechanisms, as in `.find_for_git_client`. This may lead to
       # unwanted access locks when the value provided for `password` was actually
       # a PAT, deploy token, etc.
-      def find_with_user_password(login, password, increment_failed_attempts: false)
+      def find_with_user_password(login, password, increment_failed_attempts: false, request: nil)
         # Avoid resource intensive checks if login credentials are not provided
         return unless login.present? && password.present?
 
@@ -136,31 +167,26 @@ module Gitlab
 
           break if user && !user.can_log_in_with_non_expired_password?
 
-          authenticators = []
-
-          if user
-            authenticators << Gitlab::Auth::OAuth::Provider.authentication(user, 'database')
-
-            # Add authenticators for all identities if user is not nil
-            user&.identities&.each do |identity|
-              authenticators << Gitlab::Auth::OAuth::Provider.authentication(user, identity.provider)
-            end
-          else
-            # If no user is provided, try LDAP.
-            #   LDAP users are only authenticated via LDAP
-            authenticators << Gitlab::Auth::Ldap::Authentication
-          end
-
-          authenticators.compact!
+          authenticators = password_authenticators_for_user(user)
 
           # return found user that was authenticated first for given login credentials
-          authenticated_user = authenticators.find do |auth|
-            authenticated_user = auth.login(login, password)
-            break authenticated_user if authenticated_user
+          authenticated_user, authenticator = authenticators.find do |authenticator|
+            authenticated_user = authenticator.login(login, password)
+            break authenticated_user, authenticator if authenticated_user
           end
 
           user_auth_attempt!(user, success: !!authenticated_user) if increment_failed_attempts
 
+          # Increase our visibility of authentication methods
+          if !!authenticated_user
+            # To mitigate the risk of large log volume we will log
+            # only when request is present, and use a Feature Flag with
+            # the request actor.
+            if request.present? && Feature.enabled?(:log_find_with_user_password, Feature.current_request)
+              log_authentication('Gitlab::Auth find_with_user_password succeeded', authenticated_user, authenticator, request: request)
+            end
+          end
+
           authenticated_user
         end
       end
@@ -226,16 +252,23 @@ module Gitlab
         Gitlab::Auth::Result.new(nil, project, :ci, build_authentication_abilities)
       end
 
-      def user_with_password_for_git(login, password)
-        if Feature.enabled?(:prevent_token_prefixed_password_fallback_sessionless, :instance) &&
-            password.present? &&
-            Authn::AgnosticTokenIdentifier.token?(password)
-          return
-        end
+      def user_with_password_for_git(login, password, request: nil)
+        # Prevent LDAP and database authentication attempts when password is a token
+        return if password.present? && Authn::AgnosticTokenIdentifier.token?(password)
 
-        user = find_with_user_password(login, password)
+        user = find_with_user_password(login, password, request: request)
         return unless user
 
+        if user.ldap_user? &&
+            Gitlab::CurrentSettings.password_authentication_enabled_for_git? &&
+            Gitlab::Auth::Ldap::Config.prevent_ldap_sign_in?
+          track_internal_event(
+            'authenticate_to_ldap_with_git_over_https_when_prevent_ldap_sign_in_is_enabled',
+            user: user,
+            category: name
+          )
+        end
+
         verifier = TwoFactorAuthVerifier.new(user)
 
         if user.two_factor_enabled? || verifier.two_factor_authentication_enforced?
@@ -253,7 +286,8 @@ module Gitlab
             identity = ::Gitlab::Auth::Identity.link_from_oauth_token(token)
             return if identity && !identity.valid?
 
-            user = User.id_in(token.resource_owner_id).first
+            resource_owner_id = identity&.composite? ? identity.scoped_user.id : token.resource_owner_id
+            user = User.id_in(resource_owner_id).first
             return unless user && user.can_log_in_with_non_expired_password?
 
             Gitlab::Auth::Result.new(user, nil, :oauth, abilities_for_scopes(token.scopes))
@@ -310,9 +344,10 @@ module Gitlab
           write_virtual_registry: %i[write_dependency_proxy],
           read_repository: %i[download_code],
           write_repository: %i[download_code push_code],
-          create_runner: %i[create_instance_runner create_runner],
+          create_runner: %i[create_instance_runners create_runners],
           manage_runner: %i[assign_runner update_runner delete_runner],
-          ai_workflows: %i[push_code download_code]
+          ai_workflows: %i[push_code download_code],
+          mcp: %i[execute_mcp_tool] # This ability doesn't exist yet
         }
 
         scopes.flat_map do |scope|
@@ -382,8 +417,9 @@ module Gitlab
         if build.user
           return unless build.user.can_log_in_with_non_expired_password? || bot_user_can_read_project?(build.user, build.project)
 
+          auth_context = { authentication_method: :ci_job_token, authentication_method_id: build.id }
           # If user is assigned to build, use restricted credentials of user
-          Gitlab::Auth::Result.new(build.user, build.project, :build, build_authentication_abilities)
+          Gitlab::Auth::Result.new(build.user, build.project, :build, build_authentication_abilities, auth_context)
         else
           # Otherwise use generic CI credentials (backward compatibility)
           Gitlab::Auth::Result.new(nil, build.project, :ci, build_authentication_abilities)
@@ -516,17 +552,52 @@ module Gitlab
 
       def find_build_by_token(token)
         ::Gitlab::Database::LoadBalancing::SessionMap
-          .with_sessions([::ApplicationRecord, ::Ci::ApplicationRecord]).use_primary do
+          .with_sessions.use_primary do
           ::Ci::AuthJobFinder.new(token: token).execute
         end
       end
 
+      def password_authenticators_for_user(user)
+        authenticators = []
+
+        if user
+          authenticators << Gitlab::Auth::OAuth::Provider.authentication(user, 'database')
+
+          # Add authenticators for all identities if user is not nil
+          user&.identities&.each do |identity|
+            authenticators << Gitlab::Auth::OAuth::Provider.authentication(user, identity.provider)
+          end
+        else
+          # If no user is provided, try LDAP.
+          #   LDAP users are only authenticated via LDAP
+          authenticators << Gitlab::Auth::Ldap::Authentication
+        end
+
+        authenticators.compact
+      end
+
       def user_auth_attempt!(user, success:)
         return unless user && Gitlab::Database.read_write?
         return user.unlock_access! if success
 
         user.increment_failed_attempts!
       end
+
+      def log_authentication(message, user, authenticator, request: nil)
+        # `authenticator` can be an instance or a class
+        authenticator_class_name = authenticator.is_a?(Class) ? authenticator.to_s : authenticator.class.to_s
+
+        Gitlab::AuthLogger.info(
+          message: message,
+          user_id: user.id,
+          username: user.username,
+          authenticator: authenticator_class_name,
+          remote_ip: request&.remote_ip,
+          request_method: request&.request_method,
+          path: request&.filtered_path,
+          ua: request&.user_agent
+        )
+      end
     end
   end
 end
