@@ -7,7 +7,7 @@ module Gitlab
         class Spec < ::Gitlab::Config::Entry::Node
           include ::Gitlab::Config::Entry::Configurable
 
-          ALLOWED_KEYS = %i[inputs].freeze
+          ALLOWED_KEYS = %i[inputs component].freeze
 
           validations do
             validates :config, allowed_keys: ALLOWED_KEYS
@@ -17,6 +17,11 @@ module Gitlab
             description: 'Allowed input parameters used for interpolation.',
             inherit: false,
             metadata: { composable_class: ::Gitlab::Ci::Config::Header::Input }
+
+          entry :component, Header::Component,
+            description: 'The available component context used for interpolation.',
+            inherit: false,
+            default: []
         end
       end
     end
