@@ -237,13 +237,15 @@ module Gitlab
       end
 
       # peel_tags slows down the request by a factor of 3-4
-      def list_refs(patterns = [Gitlab::Git::BRANCH_REF_PREFIX], pointing_at_oids: [], peel_tags: false, dynamic_timeout: nil)
+      def list_refs(patterns = [Gitlab::Git::BRANCH_REF_PREFIX], pointing_at_oids: [], peel_tags: false, dynamic_timeout: nil, sort_by: nil, pagination_params: nil)
         request = Gitaly::ListRefsRequest.new(
           repository: @gitaly_repo,
-          patterns: patterns,
+          patterns: patterns.map { |p| encode_binary(p) },
           pointing_at_oids: pointing_at_oids,
-          peel_tags: peel_tags
+          peel_tags: peel_tags,
+          pagination_params: pagination_params
         )
+        request.sort_by = Gitlab::GitalyClient::ListRefsSort.new(sort_by).gitaly_sort_by if sort_by
 
         timeout = dynamic_timeout || GitalyClient.fast_timeout
 
