@@ -14,9 +14,7 @@ module Gitlab
       include Gitlab::UsageDataCounters::RedisHashCounter
 
       def track_event(event_name, category: nil, additional_properties: {}, **kwargs)
-        if Feature.enabled?(:merge_additional_properties_for_snowplow, kwargs[:user])
-          extract_additional_properties!(event_name, additional_properties, kwargs)
-        end
+        extract_additional_properties!(event_name, additional_properties, kwargs)
 
         Gitlab::Tracking::EventValidator.new(event_name, additional_properties, kwargs).validate!
 
@@ -189,15 +187,7 @@ module Gitlab
         )
       end
 
-      def gitlab_sdk_client
-        app_id = ENV['GITLAB_ANALYTICS_ID']
-        host = ENV['GITLAB_ANALYTICS_URL']
-
-        return unless app_id.present? && host.present?
-
-        buffer_size = Feature.enabled?(:internal_events_batching) ? SNOWPLOW_EMITTER_BUFFER_SIZE : DEFAULT_BUFFER_SIZE
-        GitlabSDK::Client.new(app_id: app_id, host: host, buffer_size: buffer_size)
-      end
+      def gitlab_sdk_client; end
       strong_memoize_attr :gitlab_sdk_client
 
       def base_additional_properties_keys
