@@ -2,17 +2,11 @@
 
 module Gitlab
   module Timeless
-    def self.timeless(model, &block)
+    def self.timeless(model)
       original_record_timestamps = model.record_timestamps
       model.record_timestamps = false
 
-      # negative arity means arguments are optional
-      if block.arity == 1 || block.arity < 0
-        block.call(model)
-      else
-        block.call
-      end
-
+      yield model
     ensure
       model.record_timestamps = original_record_timestamps
     end
