@@ -10,7 +10,9 @@ import (
 	"time"
 
 	"github.com/sirupsen/logrus"
+	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+	"gitlab.com/gitlab-org/labkit/correlation"
 
 	apipkg "gitlab.com/gitlab-org/gitlab/workhorse/internal/api"
 	"gitlab.com/gitlab-org/gitlab/workhorse/internal/config"
@@ -20,8 +22,9 @@ import (
 )
 
 const (
-	geoProxyEndpoint = "/api/v4/geo/proxy"
-	testDocumentRoot = "testdata/public"
+	geoProxyEndpoint             = "/api/v4/geo/proxy"
+	testDocumentRoot             = "testdata/public"
+	geoProxyDisabledResponseBody = `{"geo_enabled":false}`
 )
 
 type testCase struct {
@@ -30,6 +33,20 @@ type testCase struct {
 	expectedResponse string
 }
 
+type testCasePost struct {
+	test        testCase
+	contentType string
+	body        io.Reader
+}
+
+type testCaseRequest struct {
+	desc            string
+	method          string
+	path            string
+	headers         map[string]string
+	expectedHeaders map[string]string
+}
+
 func TestMain(m *testing.M) {
 	// Secret should be configured before any Geo API poll happens to prevent
 	// race conditions where the first API call happens without a secret path
@@ -43,7 +60,8 @@ func TestRouting(t *testing.T) {
 		handler := http.HandlerFunc(func(w http.ResponseWriter, _ *http.Request) {
 			io.WriteString(w, regex)
 		})
-		return u.route("", regex, handler)
+		metadata := routeMetadata{regex, "", ""}
+		return u.route("", metadata, handler)
 	}
 
 	const (
@@ -58,7 +76,7 @@ func TestRouting(t *testing.T) {
 			handle(u, quxbaz),
 			handle(u, main),
 		}
-	}, nil)
+	}, nil, nil)
 	ts := httptest.NewServer(u)
 	defer ts.Close()
 
@@ -92,21 +110,19 @@ func TestPollGeoProxyApiStopsWhenExplicitlyDisabled(t *testing.T) {
 }
 
 func TestPollGeoProxyApiStopsWhenGeoNotEnabled(t *testing.T) {
-	remoteServer, rsDeferredClose := startRemoteServer("Geo primary")
-	defer rsDeferredClose()
+	remoteServer := startRemoteServer(t)
 
-	geoProxyEndpointResponseBody := `{"geo_enabled":false}`
-	railsServer, deferredClose := startRailsServer("Local Rails server", &geoProxyEndpointResponseBody)
-	defer deferredClose()
+	response := geoProxyDisabledResponseBody
+	railsServer := startRailsServer(t, &response)
 
 	cfg := newUpstreamConfig(railsServer.URL)
 	roundTripper := roundtripper.NewBackendRoundTripper(cfg.Backend, "", 1*time.Minute, true)
-	remoteServerUrl := helper.URLMustParse(remoteServer.URL)
+	remoteServerURL := helper.URLMustParse(remoteServer.URL)
 
 	up := upstream{
 		Config:                *cfg,
 		RoundTripper:          roundTripper,
-		APIClient:             apipkg.NewAPI(remoteServerUrl, "", roundTripper),
+		APIClient:             apipkg.NewAPI(remoteServerURL, "", roundTripper),
 		enableGeoProxyFeature: true,
 		geoProxyPollSleep:     func(time.Duration) {},
 		geoPollerDone:         make(chan struct{}),
@@ -126,15 +142,12 @@ func TestPollGeoProxyApiStopsWhenGeoNotEnabled(t *testing.T) {
 func TestGeoProxyFeatureDisabledOnGeoSecondarySite(t *testing.T) {
 	// We could just not set up the primary, but then we'd have to assert
 	// that the internal API call isn't made. This is easier.
-	remoteServer, rsDeferredClose := startRemoteServer("Geo primary")
-	defer rsDeferredClose()
+	remoteServer := startRemoteServer(t)
 
 	geoProxyEndpointResponseBody := fmt.Sprintf(`{"geo_enabled":true,"geo_proxy_url":"%v"}`, remoteServer.URL)
-	railsServer, deferredClose := startRailsServer("Local Rails server", &geoProxyEndpointResponseBody)
-	defer deferredClose()
+	railsServer := startRailsServer(t, &geoProxyEndpointResponseBody)
 
-	ws, wsDeferredClose, _ := startWorkhorseServer(railsServer.URL, false)
-	defer wsDeferredClose()
+	ws, _ := startWorkhorseServer(t, railsServer.URL, false)
 
 	testCases := []testCase{
 		{"jobs request is served locally", "/api/v4/jobs/request", "Local Rails server received request to path /api/v4/jobs/request"},
@@ -147,7 +160,7 @@ func TestGeoProxyFeatureDisabledOnGeoSecondarySite(t *testing.T) {
 
 func TestGeoProxyFeatureEnabledOnGeoSecondarySite(t *testing.T) {
 	testCases := []testCase{
-		{"push from secondary is forwarded", "/-/push_from_secondary/foo/bar.git/info/refs", "Geo primary received request to path /-/push_from_secondary/foo/bar.git/info/refs"},
+		{"request from secondary is forwarded", "/-/from_secondary/foo/bar.git/info/refs", "Geo primary received request to path /-/from_secondary/foo/bar.git/info/refs"},
 		{"LFS files are served locally", "/group/project.git/gitlab-lfs/objects/37446575700829a11278ad3a550f244f45d5ae4fe1552778fa4f041f9eaeecf6", "Local Rails server received request to path /group/project.git/gitlab-lfs/objects/37446575700829a11278ad3a550f244f45d5ae4fe1552778fa4f041f9eaeecf6"},
 		{"jobs request is forwarded", "/api/v4/jobs/request", "Geo primary received request to path /api/v4/jobs/request"},
 		{"health check is served locally", "/-/health", "Local Rails server received request to path /-/health"},
@@ -159,12 +172,10 @@ func TestGeoProxyFeatureEnabledOnGeoSecondarySite(t *testing.T) {
 
 // This test can be removed when the environment variable `GEO_SECONDARY_PROXY` is removed
 func TestGeoProxyFeatureDisabledOnNonGeoSecondarySite(t *testing.T) {
-	geoProxyEndpointResponseBody := `{"geo_enabled":false}`
-	railsServer, deferredClose := startRailsServer("Local Rails server", &geoProxyEndpointResponseBody)
-	defer deferredClose()
+	response := geoProxyDisabledResponseBody
+	railsServer := startRailsServer(t, &response)
 
-	ws, wsDeferredClose, _ := startWorkhorseServer(railsServer.URL, false)
-	defer wsDeferredClose()
+	ws, _ := startWorkhorseServer(t, railsServer.URL, false)
 
 	testCases := []testCase{
 		{"LFS files are served locally", "/group/project.git/gitlab-lfs/objects/37446575700829a11278ad3a550f244f45d5ae4fe1552778fa4f041f9eaeecf6", "Local Rails server received request to path /group/project.git/gitlab-lfs/objects/37446575700829a11278ad3a550f244f45d5ae4fe1552778fa4f041f9eaeecf6"},
@@ -177,12 +188,10 @@ func TestGeoProxyFeatureDisabledOnNonGeoSecondarySite(t *testing.T) {
 }
 
 func TestGeoProxyFeatureEnabledOnNonGeoSecondarySite(t *testing.T) {
-	geoProxyEndpointResponseBody := `{"geo_enabled":false}`
-	railsServer, deferredClose := startRailsServer("Local Rails server", &geoProxyEndpointResponseBody)
-	defer deferredClose()
+	response := geoProxyDisabledResponseBody
+	railsServer := startRailsServer(t, &response)
 
-	ws, wsDeferredClose, _ := startWorkhorseServer(railsServer.URL, true)
-	defer wsDeferredClose()
+	ws, _ := startWorkhorseServer(t, railsServer.URL, true)
 
 	testCases := []testCase{
 		{"LFS files are served locally", "/group/project.git/gitlab-lfs/objects/37446575700829a11278ad3a550f244f45d5ae4fe1552778fa4f041f9eaeecf6", "Local Rails server received request to path /group/project.git/gitlab-lfs/objects/37446575700829a11278ad3a550f244f45d5ae4fe1552778fa4f041f9eaeecf6"},
@@ -196,11 +205,9 @@ func TestGeoProxyFeatureEnabledOnNonGeoSecondarySite(t *testing.T) {
 
 func TestGeoProxyFeatureEnabledButWithAPIError(t *testing.T) {
 	geoProxyEndpointResponseBody := "Invalid response"
-	railsServer, deferredClose := startRailsServer("Local Rails server", &geoProxyEndpointResponseBody)
-	defer deferredClose()
+	railsServer := startRailsServer(t, &geoProxyEndpointResponseBody)
 
-	ws, wsDeferredClose, _ := startWorkhorseServer(railsServer.URL, true)
-	defer wsDeferredClose()
+	ws, _ := startWorkhorseServer(t, railsServer.URL, true)
 
 	testCases := []testCase{
 		{"LFS files are served locally", "/group/project.git/gitlab-lfs/objects/37446575700829a11278ad3a550f244f45d5ae4fe1552778fa4f041f9eaeecf6", "Local Rails server received request to path /group/project.git/gitlab-lfs/objects/37446575700829a11278ad3a550f244f45d5ae4fe1552778fa4f041f9eaeecf6"},
@@ -213,18 +220,15 @@ func TestGeoProxyFeatureEnabledButWithAPIError(t *testing.T) {
 }
 
 func TestGeoProxyFeatureEnablingAndDisabling(t *testing.T) {
-	remoteServer, rsDeferredClose := startRemoteServer("Geo primary")
-	defer rsDeferredClose()
+	remoteServer := startRemoteServer(t)
 
 	geoProxyEndpointEnabledResponseBody := fmt.Sprintf(`{"geo_enabled":true,"geo_proxy_url":"%v"}`, remoteServer.URL)
 	geoProxyEndpointDisabledResponseBody := `{"geo_enabled":true}`
 	geoProxyEndpointResponseBody := geoProxyEndpointEnabledResponseBody
 
-	railsServer, deferredClose := startRailsServer("Local Rails server", &geoProxyEndpointResponseBody)
-	defer deferredClose()
+	railsServer := startRailsServer(t, &geoProxyEndpointResponseBody)
 
-	ws, wsDeferredClose, waitForNextApiPoll := startWorkhorseServer(railsServer.URL, true)
-	defer wsDeferredClose()
+	ws, waitForNextAPIPoll := startWorkhorseServer(t, railsServer.URL, true)
 
 	testCasesLocal := []testCase{
 		{"LFS files are served locally", "/group/project.git/gitlab-lfs/objects/37446575700829a11278ad3a550f244f45d5ae4fe1552778fa4f041f9eaeecf6", "Local Rails server received request to path /group/project.git/gitlab-lfs/objects/37446575700829a11278ad3a550f244f45d5ae4fe1552778fa4f041f9eaeecf6"},
@@ -234,7 +238,7 @@ func TestGeoProxyFeatureEnablingAndDisabling(t *testing.T) {
 	}
 
 	testCasesProxied := []testCase{
-		{"push from secondary is forwarded", "/-/push_from_secondary/foo/bar.git/info/refs", "Geo primary received request to path /-/push_from_secondary/foo/bar.git/info/refs"},
+		{"request from secondary is forwarded", "/-/from_secondary/foo/bar.git/info/refs", "Geo primary received request to path /-/from_secondary/foo/bar.git/info/refs"},
 		{"LFS files are served locally", "/group/project.git/gitlab-lfs/objects/37446575700829a11278ad3a550f244f45d5ae4fe1552778fa4f041f9eaeecf6", "Local Rails server received request to path /group/project.git/gitlab-lfs/objects/37446575700829a11278ad3a550f244f45d5ae4fe1552778fa4f041f9eaeecf6"},
 		{"jobs request is forwarded", "/api/v4/jobs/request", "Geo primary received request to path /api/v4/jobs/request"},
 		{"health check is served locally", "/-/health", "Local Rails server received request to path /-/health"},
@@ -247,13 +251,13 @@ func TestGeoProxyFeatureEnablingAndDisabling(t *testing.T) {
 	// Disable proxying and run tests. It's safe to write to
 	// geoProxyEndpointResponseBody because the polling goroutine is blocked.
 	geoProxyEndpointResponseBody = geoProxyEndpointDisabledResponseBody
-	waitForNextApiPoll()
+	waitForNextAPIPoll()
 
 	runTestCases(t, ws, testCasesLocal)
 
 	// Re-enable proxying and run tests
 	geoProxyEndpointResponseBody = geoProxyEndpointEnabledResponseBody
-	waitForNextApiPoll()
+	waitForNextAPIPoll()
 
 	runTestCases(t, ws, testCasesProxied)
 }
@@ -262,8 +266,8 @@ func TestGeoProxyUpdatesExtraDataWhenChanged(t *testing.T) {
 	var expectedGeoProxyExtraData string
 
 	remoteServer := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
-		require.Equal(t, "1", r.Header.Get("Gitlab-Workhorse-Geo-Proxy"), "custom proxy header")
-		require.Equal(t, expectedGeoProxyExtraData, r.Header.Get("Gitlab-Workhorse-Geo-Proxy-Extra-Data"), "custom extra data header")
+		assert.Equal(t, "1", r.Header.Get("Gitlab-Workhorse-Geo-Proxy"), "custom proxy header")
+		assert.Equal(t, expectedGeoProxyExtraData, r.Header.Get("Gitlab-Workhorse-Geo-Proxy-Extra-Data"), "custom extra data header")
 		w.WriteHeader(http.StatusOK)
 	}))
 	defer remoteServer.Close()
@@ -274,27 +278,37 @@ func TestGeoProxyUpdatesExtraDataWhenChanged(t *testing.T) {
 	geoProxyEndpointResponseBody := geoProxyEndpointExtraData1
 	expectedGeoProxyExtraData = "data1"
 
-	railsServer, deferredClose := startRailsServer("Local Rails server", &geoProxyEndpointResponseBody)
-	defer deferredClose()
+	railsServer := startRailsServer(t, &geoProxyEndpointResponseBody)
 
-	ws, wsDeferredClose, waitForNextApiPoll := startWorkhorseServer(railsServer.URL, true)
-	defer wsDeferredClose()
+	ws, waitForNextAPIPoll := startWorkhorseServer(t, railsServer.URL, true)
 
-	http.Get(ws.URL)
+	res, err := http.Get(ws.URL)
+	if err != nil {
+		fmt.Printf("Error: %v", err)
+	}
+	defer res.Body.Close()
 
 	// Verify that the expected header changes after next updated poll.
 	geoProxyEndpointResponseBody = geoProxyEndpointExtraData2
 	expectedGeoProxyExtraData = "data2"
-	waitForNextApiPoll()
+	waitForNextAPIPoll()
 
-	http.Get(ws.URL)
+	res, err = http.Get(ws.URL)
+	if err != nil {
+		fmt.Printf("Error: %v", err)
+	}
+	defer res.Body.Close()
 
 	// Validate that non-existing extra data results in empty header
 	geoProxyEndpointResponseBody = geoProxyEndpointExtraData3
 	expectedGeoProxyExtraData = ""
-	waitForNextApiPoll()
+	waitForNextAPIPoll()
 
-	http.Get(ws.URL)
+	res, err = http.Get(ws.URL)
+	if err != nil {
+		fmt.Printf("Error: %v", err)
+	}
+	defer res.Body.Close()
 }
 
 func TestGeoProxySetsCustomHeader(t *testing.T) {
@@ -310,20 +324,22 @@ func TestGeoProxySetsCustomHeader(t *testing.T) {
 	for _, tc := range testCases {
 		t.Run(tc.desc, func(t *testing.T) {
 			remoteServer := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
-				require.Equal(t, "1", r.Header.Get("Gitlab-Workhorse-Geo-Proxy"), "custom proxy header")
-				require.Equal(t, tc.extraData, r.Header.Get("Gitlab-Workhorse-Geo-Proxy-Extra-Data"), "custom proxy extra data header")
+				assert.Equal(t, "1", r.Header.Get("Gitlab-Workhorse-Geo-Proxy"), "custom proxy header")
+				assert.Equal(t, tc.extraData, r.Header.Get("Gitlab-Workhorse-Geo-Proxy-Extra-Data"), "custom proxy extra data header")
 				w.WriteHeader(http.StatusOK)
 			}))
 			defer remoteServer.Close()
 
 			geoProxyEndpointResponseBody := fmt.Sprintf(tc.json, remoteServer.URL)
-			railsServer, deferredClose := startRailsServer("Local Rails server", &geoProxyEndpointResponseBody)
-			defer deferredClose()
+			railsServer := startRailsServer(t, &geoProxyEndpointResponseBody)
 
-			ws, wsDeferredClose, _ := startWorkhorseServer(railsServer.URL, true)
-			defer wsDeferredClose()
+			ws, _ := startWorkhorseServer(t, railsServer.URL, true)
 
-			http.Get(ws.URL)
+			res, err := http.Get(ws.URL)
+			if err != nil {
+				fmt.Printf("Error: %v", err)
+			}
+			defer res.Body.Close()
 		})
 	}
 }
@@ -345,20 +361,79 @@ func runTestCases(t *testing.T, ws *httptest.Server, testCases []testCase) {
 	}
 }
 
+func runTestCasesPost(t *testing.T, ws *httptest.Server, testCases []testCasePost) {
+	t.Helper()
+	for _, tc := range testCases {
+		t.Run(tc.test.desc, func(t *testing.T) {
+			resp, err := http.Post(ws.URL+tc.test.path, tc.contentType, tc.body)
+			require.NoError(t, err)
+			defer resp.Body.Close()
+
+			body, err := io.ReadAll(resp.Body)
+			require.NoError(t, err)
+
+			require.Equal(t, 200, resp.StatusCode, "response code")
+			require.Equal(t, tc.test.expectedResponse, string(body))
+		})
+	}
+}
+
+func runTestCasesRequest(t *testing.T, ws *httptest.Server, testCases []testCaseRequest) {
+	t.Helper()
+	for _, tc := range testCases {
+		t.Run(tc.desc, func(t *testing.T) {
+			client := http.Client{}
+			request, err := http.NewRequest(tc.method, ws.URL+tc.path, nil)
+			require.NoError(t, err)
+			for key, value := range tc.headers {
+				request.Header.Set(key, value)
+			}
+
+			resp, err := client.Do(request)
+			require.NoError(t, err)
+			defer resp.Body.Close()
+
+			require.Equal(t, 200, resp.StatusCode, "response code")
+			for key, value := range tc.expectedHeaders {
+				require.Equal(t, resp.Header.Get(key), value, fmt.Sprint("response header ", key))
+			}
+		})
+	}
+}
+
 func runTestCasesWithGeoProxyEnabled(t *testing.T, testCases []testCase) {
-	remoteServer, rsDeferredClose := startRemoteServer("Geo primary")
-	defer rsDeferredClose()
+	remoteServer := startRemoteServer(t)
 
 	geoProxyEndpointResponseBody := fmt.Sprintf(`{"geo_enabled":true,"geo_proxy_url":"%v"}`, remoteServer.URL)
-	railsServer, deferredClose := startRailsServer("Local Rails server", &geoProxyEndpointResponseBody)
-	defer deferredClose()
+	railsServer := startRailsServer(t, &geoProxyEndpointResponseBody)
 
-	ws, wsDeferredClose, _ := startWorkhorseServer(railsServer.URL, true)
-	defer wsDeferredClose()
+	ws, _ := startWorkhorseServer(t, railsServer.URL, true)
 
 	runTestCases(t, ws, testCases)
 }
 
+func runTestCasesWithGeoProxyEnabledPost(t *testing.T, testCases []testCasePost) {
+	remoteServer := startRemoteServer(t)
+
+	geoProxyEndpointResponseBody := fmt.Sprintf(`{"geo_enabled":true,"geo_proxy_url":"%v"}`, remoteServer.URL)
+	railsServer := startRailsServer(t, &geoProxyEndpointResponseBody)
+
+	ws, _ := startWorkhorseServer(t, railsServer.URL, true)
+
+	runTestCasesPost(t, ws, testCases)
+}
+
+func runTestCasesWithGeoProxyEnabledRequest(t *testing.T, testCases []testCaseRequest) {
+	remoteServer := startRemoteServer(t)
+
+	geoProxyEndpointResponseBody := fmt.Sprintf(`{"geo_enabled":true,"geo_proxy_url":"%v"}`, remoteServer.URL)
+	railsServer := startRailsServer(t, &geoProxyEndpointResponseBody)
+
+	ws, _ := startWorkhorseServer(t, railsServer.URL, true)
+
+	runTestCasesRequest(t, ws, testCases)
+}
+
 func newUpstreamConfig(authBackend string) *config.Config {
 	return &config.Config{
 		Version:            "123",
@@ -368,18 +443,22 @@ func newUpstreamConfig(authBackend string) *config.Config {
 	}
 }
 
-func startRemoteServer(serverName string) (*httptest.Server, func()) {
+func startRemoteServer(t *testing.T) *httptest.Server {
 	ts := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
-		body := serverName + " received request to path " + r.URL.Path
+		body := "Geo primary received request to path " + r.URL.Path
 
 		w.WriteHeader(200)
 		fmt.Fprint(w, body)
 	}))
 
-	return ts, ts.Close
+	t.Cleanup(func() {
+		ts.Close()
+	})
+
+	return ts
 }
 
-func startRailsServer(railsServerName string, geoProxyEndpointResponseBody *string) (*httptest.Server, func()) {
+func startRailsServer(t *testing.T, geoProxyEndpointResponseBody *string) *httptest.Server {
 	ts := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
 		var body string
 
@@ -387,17 +466,21 @@ func startRailsServer(railsServerName string, geoProxyEndpointResponseBody *stri
 			w.Header().Set("Content-Type", "application/vnd.gitlab-workhorse+json")
 			body = *geoProxyEndpointResponseBody
 		} else {
-			body = railsServerName + " received request to path " + r.URL.Path
+			body = "Local Rails server received request to path " + r.URL.Path
 		}
 
 		w.WriteHeader(200)
 		fmt.Fprint(w, body)
 	}))
 
-	return ts, ts.Close
+	t.Cleanup(func() {
+		ts.Close()
+	})
+
+	return ts
 }
 
-func startWorkhorseServer(railsServerURL string, enableGeoProxyFeature bool) (*httptest.Server, func(), func()) {
+func startWorkhorseServer(t *testing.T, railsServerURL string, enableGeoProxyFeature bool) (*httptest.Server, func()) {
 	geoProxySleepC := make(chan struct{})
 	geoProxySleep := func(time.Duration) {
 		geoProxySleepC <- struct{}{}
@@ -415,16 +498,20 @@ func startWorkhorseServer(railsServerURL string, enableGeoProxyFeature bool) (*h
 		configureRoutes(u)
 	}
 	cfg := newUpstreamConfig(railsServerURL)
-	upstreamHandler := newUpstream(*cfg, logrus.StandardLogger(), myConfigureRoutes, nil)
+	upstreamHandler := newUpstream(*cfg, logrus.StandardLogger(), myConfigureRoutes, nil, nil)
 	ws := httptest.NewServer(upstreamHandler)
 
-	waitForNextApiPoll := func() {}
+	t.Cleanup(func() {
+		ws.Close()
+	})
+
+	waitForNextAPIPoll := func() {}
 
 	if enableGeoProxyFeature {
 		// Wait for geoProxySleep to be entered for the first time
 		<-geoProxySleepC
 
-		waitForNextApiPoll = func() {
+		waitForNextAPIPoll = func() {
 			// Cause geoProxySleep to return
 			geoProxySleepC <- struct{}{}
 
@@ -433,5 +520,125 @@ func startWorkhorseServer(railsServerURL string, enableGeoProxyFeature bool) (*h
 		}
 	}
 
-	return ws, ws.Close, waitForNextApiPoll
+	return ws, waitForNextAPIPoll
+}
+
+func TestAdoptCfRayHeaderCorrelation(t *testing.T) {
+	// Test that when AdoptCfRayHeader is enabled with PropagateCorrelationID,
+	// the correlation middleware adopts the Cf-Ray header
+	tests := []struct {
+		name                   string
+		propagateCorrelationID bool
+		adoptCfRayHeader       bool
+		cfRayHeader            string
+		xRequestID             string
+		expectedHeader         string
+	}{
+		{
+			name:                   "adopt Cf-Ray when both flags enabled and Cf-Ray present",
+			propagateCorrelationID: true,
+			adoptCfRayHeader:       true,
+			cfRayHeader:            "test-cf-ray-id",
+			xRequestID:             "test-request-id",
+			expectedHeader:         "test-cf-ray-id",
+		},
+		{
+			name:                   "fallback to X-Request-ID when Cf-Ray absent",
+			propagateCorrelationID: true,
+			adoptCfRayHeader:       true,
+			cfRayHeader:            "",
+			xRequestID:             "test-request-id",
+			expectedHeader:         "test-request-id",
+		},
+		{
+			name:                   "use X-Request-ID when adoptCfRayHeader disabled",
+			propagateCorrelationID: true,
+			adoptCfRayHeader:       false,
+			cfRayHeader:            "test-cf-ray-id",
+			xRequestID:             "test-request-id",
+			expectedHeader:         "test-request-id",
+		},
+		{
+			name:                   "generate new ID when propagation disabled",
+			propagateCorrelationID: false,
+			adoptCfRayHeader:       true,
+			cfRayHeader:            "test-cf-ray-id",
+			xRequestID:             "test-request-id",
+			expectedHeader:         "", // Will be generated
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			cfg := newUpstreamConfig("http://localhost:8080")
+			cfg.PropagateCorrelationID = tt.propagateCorrelationID
+			cfg.AdoptCfRayHeader = tt.adoptCfRayHeader
+
+			// Create a test handler that captures the correlation ID from context
+			var capturedCorrelationID string
+			handler := http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+				capturedCorrelationID = correlation.ExtractFromContext(r.Context())
+				w.WriteHeader(http.StatusOK)
+			})
+
+			upstreamHandler := newUpstream(*cfg, logrus.StandardLogger(), func(u *upstream) {
+				u.Routes = []routeEntry{
+					u.route("", routeMetadata{}, handler),
+				}
+			}, nil, nil)
+
+			// Create a test request
+			req := httptest.NewRequest("GET", "/", nil)
+			if tt.cfRayHeader != "" {
+				req.Header.Set("Cf-Ray", tt.cfRayHeader)
+			}
+			if tt.xRequestID != "" {
+				req.Header.Set("X-Request-ID", tt.xRequestID)
+			}
+
+			// Execute the request
+			rr := httptest.NewRecorder()
+			upstreamHandler.ServeHTTP(rr, req)
+
+			// Verify the captured correlation ID
+			if tt.expectedHeader != "" {
+				require.Equal(t, tt.expectedHeader, capturedCorrelationID)
+			} else {
+				// When not propagating, a new ID is generated
+				require.NotEmpty(t, capturedCorrelationID)
+				require.NotEqual(t, tt.cfRayHeader, capturedCorrelationID)
+				require.NotEqual(t, tt.xRequestID, capturedCorrelationID)
+			}
+		})
+	}
+}
+
+func TestFixRemoteAddr(t *testing.T) {
+	testCases := []struct {
+		initial   string
+		forwarded string
+		expected  string
+	}{
+		{initial: "@", forwarded: "", expected: "127.0.0.1:0"},
+		{initial: "@", forwarded: "18.245.0.1", expected: "18.245.0.1:0"},
+		{initial: "@", forwarded: "127.0.0.1", expected: "127.0.0.1:0"},
+		{initial: "@", forwarded: "192.168.0.1", expected: "127.0.0.1:0"},
+		{initial: "192.168.1.1:0", forwarded: "", expected: "192.168.1.1:0"},
+		{initial: "192.168.1.1:0", forwarded: "18.245.0.1", expected: "18.245.0.1:0"},
+	}
+
+	for _, tc := range testCases {
+		req, err := http.NewRequest("POST", "unix:///tmp/test.socket/info/refs", nil)
+		require.NoError(t, err)
+
+		req.RemoteAddr = tc.initial
+
+		if tc.forwarded != "" {
+			req.Header.Add("X-Forwarded-For", tc.forwarded)
+		}
+
+		fixRemoteAddr(req)
+
+		require.Equal(t, tc.expected, req.RemoteAddr)
+	}
 }
